==> zither-1.1.14.tar.gz: zither_prj/sitar/scan_urlport.c <==

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<unistd.h>
#include<fcntl.h>

#include<errno.h>

#include"sitar_scan.h"
#include"sitar_com.h"
#include<almemsys/almemsys.h>
#include"jpreturn.h"



/* 1-> port No' 2-> Ipsddr 3-> return data,, return status code 0=open or=close
int scan_url_connect(int, char *, char *, int);
unsigned long resolvename(char *, char *);
*/

int urldompass(char *, char *, char *, int); 
char * set_request(char *, char *, char *);
char * check_recvdata(char *, char *, int *);

 
int urldompass(char * url_data, char * url_dom, char * url_pass, int datain)
{
int ret;
int count, count2;

if(datain < BUF_SLEN){

   if(NULL == strchr(url_data, '/')){
      self_memcpy(url_dom, url_data, BUF_SLEN);
      url_pass[0] = (char)0x00;
      ret = 1;
      return(ret);
      }

   else {
      for(count = 0; datain >= count; count++){
         if(url_data[count] != '/'){
            url_dom[count] = url_data[count];
            }
         else {
            url_dom[count] = (char)0x00;
            break;
            }
         }
      count++;
      if(url_data[count]!=(char)0x00){
         for(count2 = 0; datain >= count; count++){
            if(url_data[count] != (char)0x00){
               url_pass[count2] = url_data[count];
               }
            else {
               url_pass[count2] = (char)0x00;
               break;
               }
            count2++;
            } 
         }
      else {
         url_pass[0] = (char)0x00;
         ret = 1;
         return(ret);
         }
      ret = 1;
      return(ret);
      }

   }
ret = -1;
return(ret);
} 




char * resolvename(char * backdata, char *name, int timeout, unsigned long * ret)
{
char * recdata;
char * sendcom;

if((recdata = (char *)calloc(BUF_DEFLEN, sizeof(char)))==NULL){ exit(1);}
if((sendcom = (char *)calloc(BUF_DEFLEN, sizeof(char)))==NULL){ exit(1);}

sendcom = safe_sprintf(sendcom, BUF_MAX,"%s %s",
  "/usr/local/zither/bin/resolv", name);

recdata = execom(sendcom, recdata, timeout, 1);

#ifdef RESOLV_DEBUG
printf("RESOLV: resolvname(2): sendcom %s resolvname %s\n", sendcom, recdata);
#endif

* ret = 0;
if(0 == memcmp(recdata, COM_1, 20)){
   backdata = safe_memcpy(backdata, "SOCK_10", BUF_MAX);
   #ifdef RESOLV_DEBUG
   fprintf(stderr,"RESOLV: resolvname(3): %s %s\n", SOCK_10, name);
   #endif
   * ret = 0;
   }
else if(0 == memcmp(recdata, "0.0.0.0", 7)){
   backdata = safe_memcpy(backdata, "SOCK_2", BUF_MAX);
   #ifdef RESOLV_DEBUG
   fprintf(stderr,"RESOLV: resolvname(4): %s %s\n", SOCK_2, name);
   #endif
   * ret = 0;
   }
else{
   backdata = safe_memcpy(backdata, "SOCK_11", BUF_MAX);
   #ifdef RESOLV_DEBUG
   fprintf(stderr,"RESOLV: resolvname(5): %s %s\n", SOCK_11, name);
   #endif
   * ret = (unsigned long)inet_addr(recdata);
   }

free(recdata);
free(sendcom);

return(backdata);
}





char * set_request(char * backdata, char * url_dom, char * url_pass)
{

backdata = safe_sprintf(backdata, BUF_MAX,
"HEAD /%s HTTP/1.0\nHost: %s\nUser-Agent: zither-1.1.1(monitorring server)\nPragma: no-cache\nCache-Control: no-cache\r\n\r\n", url_pass, url_dom);

#ifdef URL_BUG
printf("URL: set_request(): SEND HEADERS\n");
printf("%s", backdata);
#endif

return backdata;
}



char * check_recvdata(char * recvdata, char * data, int * ret)
{
int dret;
int count = 0;
int inc = 0;
char * swap;

dret = 0;

if((swap = (char *)calloc(BUF_DEFLEN, sizeof(char)))==NULL){ exit(1);}

* ret = -1;
for(count = 0; inc == 0; count++){
   data[count] = recvdata[count];
   if(data[count] == '\n'){
      inc = 1;
      }
   if(count >= BUF_LEN){
      inc = 1;
      }
   }
if(count > 1){
   count = count - 2;
   data[count] = (char)0x00;

   #ifdef URL_BUG
   printf("-1--%s---\n", data);
   #endif
   swap = delim_get(data, swap, ' ', &dret, 1);
   #ifdef URL_BUG
   printf("-2--%s---\n", swap);
   #endif
   if(3 == (int)strlen(swap)){
      if('2' == swap[0]){
         #ifdef URL_BUG
         printf("-200--%c---\n", swap[0]);
         #endif
         * ret = 2;
         }
      else if('3' == swap[0]){
         #ifdef URL_BUG
         printf("-300--%c---\n", swap[0]);
         #endif
         * ret = 3;
         }
      else if('1' == swap[0]){
         #ifdef URL_BUG
         printf("-100--%c---\n", swap[0]);
         #endif
         * ret = -1;
         }
      else if('4' == swap[0]){
         #ifdef URL_BUG
         printf("-400--%c---\n", swap[0]);
         #endif
         * ret = -1;
         }
      else if('5' == swap[0]){
         #ifdef URL_BUG
         printf("-500--%c---\n", swap[0]);
         #endif
         * ret = -1;
         }
      else{
         #ifdef URL_BUG
         printf("-other--%c---\n", swap[0]);
         #endif
         * ret = -1;
         }
      }
   else{
      * ret = -1;
      }
   }

else{
   data = safe_sprintf(data, BUF_MAX, "%s", URL_1);
   * ret = -1;
   } 

free(swap);

return(data);
}




/*----------------- SCAN TCP CONNECT --------------------*/
char * scan_url_connect(int port, char * url_data, char * backdata, int timeout, int * pst)
{
int sk, re, stat, ret, retval, dcount, arglen;
unsigned long resoip;
char ipa[IP_LEN];
char dom[1024];
char pass[1024];
char * ip_addr = ipa;
char * url_dom = dom;
char * url_pass = pass;
struct sockaddr_in my_addr;
fd_set rfds;
struct timeval tv;
char * recvdata = NULL;
char * senddata = NULL;

dcount = 0;
arglen = sizeof(int);
ret = -1;
errno = 0;

if((recvdata = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ exit(1);}
if((senddata = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ exit(1);}

stat = urldompass(url_data, url_dom, url_pass, strlen(url_data)); 

if((sk = socket(AF_INET, SOCK_STREAM, 0))==-1){
   fprintf(stderr,"scan_url URL=%s Port=%d tcp %s", url_data, port, SOCK_1);
   backdata=safe_sprintf(backdata, BUF_MAX,
    "URL=%s Port=%d tcp %s", url_data, port, SOCK_1);
   #ifdef URL_BUG
   printf("URL: scan_url_connect(1): NOT CLEATE SOCKETS %s\n", backdata);
   #endif
   * pst = -1;
   free(recvdata);
   free(senddata);
   return backdata;
   }

my_addr.sin_family = AF_INET;
my_addr.sin_port = htons(port);

backdata = resolvename(backdata, url_dom, timeout, &resoip);
my_addr.sin_addr.s_addr = resoip;


if(0 == (int)my_addr.sin_addr.s_addr || resoip == (int)NULL){
   backdata=safe_sprintf(backdata,BUF_MAX,"%s %s %d\n",
    SOCK_2, url_dom, (int)my_addr.sin_addr.s_addr);
   #ifdef URL_BUG
   printf("URL: scan_url_connect(2):  NOT RESOLV NAME %s\n", backdata);
   #endif
   close(sk);
   * pst = -1;
   free(recvdata);
   free(senddata);
   return backdata;
   }

ip_addr = inet_ntoa(my_addr.sin_addr);

#ifdef URL_BUG
printf("URL: scan_url_connect(3): %s\n", ip_addr);
#endif

// 非同期通信の開始、TCPポートにコネクション
fcntl(sk, F_SETFL, O_NONBLOCK);
re = connect(sk, (struct sockaddr *)(long)&my_addr, sizeof(my_addr));


// 速攻で繋がった場合の処理
if(re >= 0){
   // 非同期通信から同期通信に
   fcntl(sk, F_SETFL, 0);
   backdata=safe_sprintf(backdata,BUF_MAX,
    "URL=%s Port=%d %s", url_data, port, SOCK_5);
   #ifdef URL_BUG
   printf("URL: scan_url_connect(4): CONNECT SUCCESS %s\n", backdata);
   #endif
   goto notselect;
   }

// errnoが EINPROGRESS な場合
if(errno == EINPROGRESS){
   tv.tv_sec = timeout;
   tv.tv_usec = 0;
   while(1){
      FD_ZERO(&rfds);
      FD_SET(sk, &rfds);
      retval = select(sk + 1, NULL, &rfds, NULL, &tv);
      // selectの異常終了
      if(retval < 0){
         backdata=safe_sprintf(backdata,BUF_MAX,
          "URL=%s Port=%d %s %s", url_data, port, SOCK_15, SOCK_5);
         #ifdef URL_BUG
         printf("URL: scan_url_connect(4): select error BREAK- %d - %s\n", retval, backdata);
         #endif
         // 非同期通信から同期通信に
         fcntl(sk, F_SETFL, 0);
         close(sk);
         * pst = -1;
         free(recvdata);
         free(senddata);
         return backdata;
         }
      // selectのタイムアウト
      else if(retval == 0){
         backdata=safe_sprintf(backdata,BUF_MAX,
          "URL=%s Port=%d %s %s", url_data, port, SOCK_16, SOCK_5);
         #ifdef URL_BUG
         printf("URL: scan_url_connect(5): select time out BREAK- %d - %s\n", retval, backdata);
         #endif
         // 非同期通信から同期通信に
         fcntl(sk, F_SETFL, 0);
         close(sk);
         * pst = -1;
         free(recvdata);
         free(senddata);
         return backdata;
         }
      // 時間内に接続出来たのでLoopOut
      else{
         backdata=safe_sprintf(backdata,BUF_MAX,
          "URL=%s Port=%d %s", url_data, port, SOCK_3);
         #ifdef URL_BUG
         printf("URL: scan_url_connect(6): select time succes BREAK-%d- %s\n", retval, backdata);
         #endif
         break;
         }
      }


   // 非同期通信から同期通信に
   fcntl(sk, F_SETFL, 0);

   // select を省いたGOTO
   notselect :;

   // ソケットに関するペンディングエラーを戻す
   if(getsockopt(sk, SOL_SOCKET, SO_ERROR, (void *)&retval, (void *)&arglen) < 0){
      backdata=safe_sprintf(backdata,BUF_MAX,
       "URL=%s Port=%d %s", url_data, port, SOCK_13);
      #ifdef URL_BUG
      fprintf(stderr,"getsockopt(7) Errors!\n");
      #endif
      close(sk);
      * pst = -1;
      free(recvdata);
      free(senddata);
      return backdata;
      }
   else{
      // エラーがある
      if(retval){
         backdata=safe_sprintf(backdata,BUF_MAX,
          "URL=%s Port=%d %s: TCP %s", url_data, port, (retval != 0) ? strerror(retval): "0", SOCK_5);
         #ifdef URL_BUG
         printf("URL: scan_url_connect(8): 1 SO_ERROR %d %s\n", retval, backdata);
         #endif
         close(sk);
         * pst = -1;
         free(recvdata);
         free(senddata);
         return backdata;
         }
      // エラーが無い場合
      else{
         backdata=safe_sprintf(backdata,BUF_MAX,
          "URL=%s Port=%d %s", url_data, port, SOCK_3);
         senddata = set_request(senddata, url_dom, url_pass);
         #ifdef URL_BUG
         printf("URL: scan_url_connect(9): set_request*send-%s- %s\n", senddata, backdata);
         #endif

         // 再度非同期通信の開始
         fcntl(sk, F_SETFL, O_NONBLOCK);

         // HEADメソッドの送信
         send(sk, senddata, strlen(senddata), 0);

         tv.tv_sec = timeout;
         tv.tv_usec = 0;
         while(1){
            FD_ZERO(&rfds);
            FD_SET(sk, &rfds);
            retval = select(sk + 1, &rfds, NULL, NULL, &tv);
            // select の異常終了
            if(retval < 0){
               backdata=safe_sprintf(backdata,BUF_MAX,
                "URL=%s Port=%d %s %s", url_data, port, SOCK_15, URL_2);
               #ifdef URL_BUG
               printf("URL: scan_url_connect(10): select error %s\n", backdata);
               #endif 
               // 非同期通信から同期通信に
               fcntl(sk, F_SETFL, 0);
               close(sk);
               * pst = -1;
               free(recvdata);
               free(senddata);
               return backdata;
               }
            // select のタイムアウト
            else if(retval == 0){
               backdata=safe_sprintf(backdata,BUF_MAX,
                "URL=%s Port=%d %s %s", url_data, port, SOCK_16, URL_3);
               #ifdef URL_BUG
               printf("URL: scan_url_connect(11): select timed out %s\n", backdata); 
               #endif 
               // 非同期通信から同期通信に
               fcntl(sk, F_SETFL, 0);
               close(sk);
               * pst = -1;
               free(recvdata);
               free(senddata);
               return backdata;
               }
            else{
               dcount = recv(sk, recvdata, BUF_LEN, 0);
               #ifdef URL_BUG
               printf("URL: scan_url_connect(12): select succes. recv data %s<%d>\n", recvdata, dcount);
               #endif
               // 受信量が異常
               if(dcount < 1){
                  backdata=safe_sprintf(backdata,BUF_MAX,
                   "URL=%s Port=%d %s"
                   , url_data, port, URL_2);
                  ret = -1;
                  }
               // 受信量が正常
               else{
                  senddata = (char *)realloc(senddata, (dcount + 10) * sizeof(char));
                  // 200番だいであることの確認。
                  senddata = check_recvdata(recvdata, senddata, &ret);
                  if(ret < 0){
                     backdata = safe_sprintf(backdata, BUF_MAX,"%s <%s>\n", URL_1, senddata);
                     #ifdef URL_BUG
                     printf("URL: scan_url_connect(13): url check. not 200. ret<%d><%d> %s\n",
                       ret,*pst,backdata);
                     #endif 
                     }
                  else{
                     backdata = safe_sprintf(backdata, BUF_MAX,"%s <%s>\n", URL_4, senddata);
                     #ifdef URL_BUG
                     printf("URL: scan_url_connect(14): url check. all succesd. ret<%d><%d> %s\n",
                       ret,*pst,backdata);
                     #endif 
                     }
                  }
               // 非同期通信から同期通信に
               fcntl(sk, F_SETFL, 0);
               close(sk);
               * pst = ret;
               free(recvdata);
               free(senddata);
               return backdata;
               }
            }

         // 非同期通信から同期通信に
         fcntl(sk, F_SETFL, 0);
         close(sk);
         * pst = ret;
         free(recvdata);
         free(senddata);
         return backdata;
         }
      }
   }

// host IP 等が存在すらしない場合の処理
// 非同期通信から同期通信に
fcntl(sk, F_SETFL, 0);

backdata=safe_sprintf(backdata,BUF_MAX,
 "URL=%s Port=%d %s", url_data, port, SOCK_14);
#ifdef URL_BUG
printf("URL: scan_url_connect(14): not connection err. %s\n", backdata); 
#endif 

close(sk);
* pst = -1;

free(recvdata);
free(senddata);
return backdata;

}

==> zither-1.1.16.tar.gz: zither_prj/sitar/scan_urlport.c <==

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<netdb.h>
#include<unistd.h>
#include<sys/socket.h>
#include<netinet/in_systm.h>
#include<netinet/in.h>
#include<netinet/ip.h>
#include<arpa/inet.h>
#include<fcntl.h>

#include<sys/stat.h>
#include<errno.h>

#include"sitar_scan.h"
#include"sitar_com.h"
#include<almemsys/almemsys.h>
#include"jpreturn.h"



/* 1-> port No' 2-> Ipsddr 3-> return data,, return status code 0=open or=close
int scan_url_connect(int, char *, char *, int);
unsigned long resolvename(char *, char *);
*/

int urldompass(char *, char *, char *, int); 
char * set_request(char *, char *, char *);
char * check_recvdata(char *, char *, int *);

 
int urldompass(char * url_data, char * url_dom, char * url_pass, int datain)
{
int ret;
int count, count2;

if(datain < BUF_SLEN){

   if(NULL == strchr(url_data, '/')){
      self_memcpy(url_dom, url_data, BUF_SLEN);
      url_pass[0] = (char)0x00;
      ret = 1;
      return(ret);
      }

   else {
      for(count = 0; datain >= count; count++){
         if(url_data[count] != '/'){
            url_dom[count] = url_data[count];
            }
         else {
            url_dom[count] = (char)0x00;
            break;
            }
         }
      count++;
      if(url_data[count]!=(char)0x00){
         for(count2 = 0; datain >= count; count++){
            if(url_data[count] != (char)0x00){
               url_pass[count2] = url_data[count];
               }
            else {
               url_pass[count2] = (char)0x00;
               break;
               }
            count2++;
            } 
         }
      else {
         url_pass[0] = (char)0x00;
         ret = 1;
         return(ret);
         }
      ret = 1;
      return(ret);
      }

   }
ret = -1;
return(ret);
} 




char * set_request(char * backdata, char * url_dom, char * url_pass)
{

backdata = safe_sprintf(backdata, BUF_MAX,
"HEAD /%s HTTP/1.0\nHost: %s\nUser-Agent: zither-1.1.1(monitorring server)\nPragma: no-cache\nCache-Control: no-cache\r\n\r\n", url_pass, url_dom);

#ifdef URL_BUG
printf("URL: set_request(): SEND HEADERS\n");
printf("%s", backdata);
#endif

return backdata;
}



char * check_recvdata(char * recvdata, char * data, int * ret)
{
int dret;
int count = 0;
int inc = 0;
char * swap;

dret = 0;

if((swap = (char *)calloc(BUF_DEFLEN, sizeof(char)))==NULL){ exit(1);}

* ret = -1;
for(count = 0; inc == 0; count++){
   data[count] = recvdata[count];
   if(data[count] == '\n'){
      inc = 1;
      }
   if(count >= BUF_LEN){
      inc = 1;
      }
   }
if(count > 1){
   count = count - 2;
   data[count] = (char)0x00;

   #ifdef URL_BUG
   printf("-1--%s---\n", data);
   #endif
   swap = delim_get(data, swap, ' ', &dret, 1);
   #ifdef URL_BUG
   printf("-2--%s---\n", swap);
   #endif
   if(3 == (int)strlen(swap)){
      if('2' == swap[0]){
         #ifdef URL_BUG
         printf("-200--%c---\n", swap[0]);
         #endif
         * ret = 2;
         }
      else if('3' == swap[0]){
         #ifdef URL_BUG
         printf("-300--%c---\n", swap[0]);
         #endif
         * ret = 3;
         }
      else if('1' == swap[0]){
         #ifdef URL_BUG
         printf("-100--%c---\n", swap[0]);
         #endif
         * ret = -1;
         }
      else if('4' == swap[0]){
         #ifdef URL_BUG
         printf("-400--%c---\n", swap[0]);
         #endif
         * ret = -1;
         }
      else if('5' == swap[0]){
         #ifdef URL_BUG
         printf("-500--%c---\n", swap[0]);
         #endif
         * ret = -1;
         }
      else{
         #ifdef URL_BUG
         printf("-other--%c---\n", swap[0]);
         #endif
         * ret = -1;
         }
      }
   else{
      * ret = -1;
      }
   }

else{
   data = safe_sprintf(data, BUF_MAX, "%s", URL_1);
   * ret = -1;
   } 

free(swap);

return(data);
}




/*----------------- SCAN TCP CONNECT --------------------*/
char * scan_url_connect(int port, char * url_data, char * backdata, int timeout, int * pst)
{
int sk, re, stat, ret, retval, dcount, arglen;
unsigned long resoip;
char ipa[IP_LEN];
char dom[1024];
char pass[1024];
char * ip_addr = ipa;
char * url_dom = dom;
char * url_pass = pass;
struct sockaddr_in my_addr;
fd_set rfds;
struct timeval tv;
char * recvdata = NULL;
char * senddata = NULL;

dcount = 0;
arglen = sizeof(int);
ret = -1;
errno = 0;

if((recvdata = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ exit(1);}
if((senddata = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ exit(1);}

stat = urldompass(url_data, url_dom, url_pass, strlen(url_data)); 

if((sk = socket(AF_INET, SOCK_STREAM, 0))==-1){
   fprintf(stderr,"scan_url URL=%s Port=%d tcp %s", url_data, port, SOCK_1);
   backdata=safe_sprintf(backdata, BUF_MAX,
    "URL=%s Port=%d tcp %s", url_data, port, SOCK_1);
   #ifdef URL_BUG
   printf("URL: scan_url_connect(1): NOT CLEATE SOCKETS %s\n", backdata);
   #endif
   * pst = -1;
   free(recvdata);
   free(senddata);
   return backdata;
   }

my_addr.sin_family = AF_INET;
my_addr.sin_port = htons(port);

backdata = resolvename(backdata, url_dom, timeout, &resoip);
my_addr.sin_addr.s_addr = resoip;


if(0 == (int)my_addr.sin_addr.s_addr || resoip == (int)NULL){
   backdata=safe_sprintf(backdata,BUF_MAX,"%s %s %d\n",
    SOCK_2, url_dom, (int)my_addr.sin_addr.s_addr);
   #ifdef URL_BUG
   printf("URL: scan_url_connect(2):  NOT RESOLV NAME %s\n", backdata);
   #endif
   close(sk);
   * pst = -1;
   free(recvdata);
   free(senddata);
   return backdata;
   }

ip_addr = inet_ntoa(my_addr.sin_addr);

#ifdef URL_BUG
printf("URL: scan_url_connect(3): %s\n", ip_addr);
#endif

// 非同期通\Bぃ開@、TCPポートにゃEネクゃLッ@ッE
fcntl(sk, F_SETFL, O_NONBLOCK);
re = connect(sk, (struct sockaddr *)(long)&my_addr, sizeof(my_addr));


// 速攻ぃ@繋がぁvた堃G合の凃
if(re >= 0){
   // 非同期通\Bから同期通\BぃH
   fcntl(sk, F_SETFL, 0);
   backdata=safe_sprintf(backdata,BUF_MAX,
    "URL=%s Port=%d %s", url_data, port, SOCK_5);
   #ifdef URL_BUG
   printf("URL: scan_url_connect(4): CONNECT SUCCESS %s\n", backdata);
   #endif
   goto notselect;
   }

// errnoが EINPROGRESS ぃF堃G
if(errno == EINPROGRESS){
   tv.tv_sec = timeout;
   tv.tv_usec = 0;
   while(1){
      FD_ZERO(&rfds);
      FD_SET(sk, &rfds);
      retval = select(sk + 1, NULL, &rfds, NULL, &tv);
      // selectぃ畁|布N終R
      if(retval < 0){
         backdata=safe_sprintf(backdata,BUF_MAX,
          "URL=%s Port=%d %s %s", url_data, port, SOCK_15, SOCK_5);
         #ifdef URL_BUG
         printf("URL: scan_url_connect(4): select error BREAK- %d - %s\n", retval, backdata);
         #endif
         // 非同期通\Bから同期通\BぃH
         fcntl(sk, F_SETFL, 0);
         close(sk);
         * pst = -1;
         free(recvdata);
         free(senddata);
         return backdata;
         }
      // selectぃゃ\めAムめuゃ
      else if(retval == 0){
         backdata=safe_sprintf(backdata,BUF_MAX,
          "URL=%s Port=%d %s %s", url_data, port, SOCK_16, SOCK_5);
         #ifdef URL_BUG
         printf("URL: scan_url_connect(5): select time out BREAK- %d - %s\n", retval, backdata);
         #endif
         // 非同期通\Bから同期通\BぃH
         fcntl(sk, F_SETFL, 0);
         close(sk);
         * pst = -1;
         free(recvdata);
         free(senddata);
         return backdata;
         }
      // 時間内に掁E続出杁Eたのぃ@LoopOut
      else{
         backdata=safe_sprintf(backdata,BUF_MAX,
          "URL=%s Port=%d %s", url_data, port, SOCK_3);
         #ifdef URL_BUG
         printf("URL: scan_url_connect(6): select time succes BREAK-%d- %s\n", retval, backdata);
         #endif
         break;
         }
      }


   // 非同期通\Bから同期通\BぃH
   fcntl(sk, F_SETFL, 0);

   // select を省いたGOTO
   notselect :;

   // ゃXゃAットぃH閁uするペンディッEめ|ゃBッDッVを戻
   if(getsockopt(sk, SOL_SOCKET, SO_ERROR, (void *)&retval, (void *)&arglen) < 0){
      backdata=safe_sprintf(backdata,BUF_MAX,
       "URL=%s Port=%d %s", url_data, port, SOCK_13);
      #ifdef URL_BUG
      fprintf(stderr,"getsockopt(7) Errors!\n");
      #endif
      close(sk);
      * pst = -1;
      free(recvdata);
      free(senddata);
      return backdata;
      }
   else{
      // ゃBッDッVがあ
      if(retval){
         backdata=safe_sprintf(backdata,BUF_MAX,
          "URL=%s Port=%d %s: TCP %s", url_data, port, (retval != 0) ? strerror(retval): "0", SOCK_5);
         #ifdef URL_BUG
         printf("URL: scan_url_connect(8): 1 SO_ERROR %d %s\n", retval, backdata);
         #endif
         close(sk);
         * pst = -1;
         free(recvdata);
         free(senddata);
         return backdata;
         }
      // ゃBッDッVが無い堃G
      else{
         backdata=safe_sprintf(backdata,BUF_MAX,
          "URL=%s Port=%d %s", url_data, port, SOCK_3);
         senddata = set_request(senddata, url_dom, url_pass);
         #ifdef URL_BUG
         printf("URL: scan_url_connect(9): set_request*send-%s- %s\n", senddata, backdata);
         #endif

         // 再R非同期通\Bぃ開@
         fcntl(sk, F_SETFL, O_NONBLOCK);

         // HEADチBゃXッドぃ送\B
         send(sk, senddata, strlen(senddata), 0);

         tv.tv_sec = timeout;
         tv.tv_usec = 0;
         while(1){
            FD_ZERO(&rfds);
            FD_SET(sk, &rfds);
            retval = select(sk + 1, &rfds, NULL, NULL, &tv);
            // select ぃ畁|布N終R
            if(retval < 0){
               backdata=safe_sprintf(backdata,BUF_MAX,
                "URL=%s Port=%d %s %s", url_data, port, SOCK_15, URL_2);
               #ifdef URL_BUG
               printf("URL: scan_url_connect(10): select error %s\n", backdata);
               #endif 
               // 非同期通\Bから同期通\BぃH
               fcntl(sk, F_SETFL, 0);
               close(sk);
               * pst = -1;
               free(recvdata);
               free(senddata);
               return backdata;
               }
            // select ぃゃ\めAムめuゃ
            else if(retval == 0){
               backdata=safe_sprintf(backdata,BUF_MAX,
                "URL=%s Port=%d %s %s", url_data, port, SOCK_16, URL_3);
               #ifdef URL_BUG
               printf("URL: scan_url_connect(11): select timed out %s\n", backdata); 
               #endif 
               // 非同期通\Bから同期通\BぃH
               fcntl(sk, F_SETFL, 0);
               close(sk);
               * pst = -1;
               free(recvdata);
               free(senddata);
               return backdata;
               }
            else{
               dcount = recv(sk, recvdata, BUF_LEN, 0);
               #ifdef URL_BUG
               printf("URL: scan_url_connect(12): select succes. recv data %s<%d>\n", recvdata, dcount);
               #endif
               // 受\B量が畁|布N
               if(dcount < 1){
                  backdata=safe_sprintf(backdata,BUF_MAX,
                   "URL=%s Port=%d %s"
                   , url_data, port, URL_2);
                  ret = -1;
                  }
               // 受\B量が歁v布N
               else{
                  senddata = (char *)realloc(senddata, (dcount + 10) * sizeof(char));
                  // 200畃Fだいであることぃ碃R認
                  senddata = check_recvdata(recvdata, senddata, &ret);
                  if(ret < 0){
                     backdata = safe_sprintf(backdata, BUF_MAX,"%s <%s>\n", URL_1, senddata);
                     #ifdef URL_BUG
                     printf("URL: scan_url_connect(13): url check. not 200. ret<%d><%d> %s\n",
                       ret,*pst,backdata);
                     #endif 
                     }
                  else{
                     backdata = safe_sprintf(backdata, BUF_MAX,"%s <%s>\n", URL_4, senddata);
                     #ifdef URL_BUG
                     printf("URL: scan_url_connect(14): url check. all succesd. ret<%d><%d> %s\n",
                       ret,*pst,backdata);
                     #endif 
                     }
                  }
               // 非同期通\Bから同期通\BぃH
               fcntl(sk, F_SETFL, 0);
               close(sk);
               * pst = ret;
               free(recvdata);
               free(senddata);
               return backdata;
               }
            }

         // 非同期通\Bから同期通\BぃH
         fcntl(sk, F_SETFL, 0);
         close(sk);
         * pst = ret;
         free(recvdata);
         free(senddata);
         return backdata;
         }
      }
   }

// host IP 等が存在すらしない堃G合の凃
// 非同期通\Bから同期通\BぃH
fcntl(sk, F_SETFL, 0);

backdata=safe_sprintf(backdata,BUF_MAX,
 "URL=%s Port=%d %s", url_data, port, SOCK_14);
#ifdef URL_BUG
printf("URL: scan_url_connect(14): not connection err. %s\n", backdata); 
#endif 

close(sk);
* pst = -1;

free(recvdata);
free(senddata);
return backdata;

}
